//----------------------------------------------------------------------------
// 
//	file: simstat.cc
// 
//	function: gather various statistics 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sat Oct 31 01:42:12 CST 1998
// 
//	$Revision: 1.1 $ $Date: 1999/02/09 07:42:25 $
//	$Log: simstat.cc,v $
//	Revision 1.1  1999/02/09 07:42:25  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------

#include "simstat.h"

//----------------------------------------------------------------------------
//	simstat() 
//	  - Constructor
//----------------------------------------------------------------------------
simstat::simstat(REAL** buf, int cols, int rows, int max_specs=1) {

   int size = 1;	// temporarily..

   int buffer_size = size;

   buffer = new REAL** [size];

   buffer[0] = buf;

   simstat::cols = cols;
   simstat::rows = rows;
  
   simstat::max_specs = max_specs;
   num_specs = 0;

   table = new SS_Entry [max_specs];

}

//----------------------------------------------------------------------------
//	~simstat() 
//	  - Destructor
//----------------------------------------------------------------------------
simstat::~simstat() {

  delete [] buffer;

  delete [] table;

}


//----------------------------------------------------------------------------
//	sum_area() 
//	  - sum activity in rectangle (x1,y1) to (x2,y2).
//----------------------------------------------------------------------------
REAL simstat::sum_area(int x1, int y1, int x2, int y2, int buf_num=0){

  REAL sum = (REAL)0;

  for (int i=x1; i<=x2; ++i) { 
    for (int j=y1; j<=y2; ++j) {
        sum += buffer[buf_num][i][j];
    }
  }

  return sum;
}


//----------------------------------------------------------------------------
//	add_spec_circle() 
//	  - add spec for a circular area
//----------------------------------------------------------------------------
void simstat::add_spec_circle(int cx, int cy, int cradius){

  if (num_specs >= max_specs) {
	warning("simstat::add_spec_circle","max specs exceeded");
	return; 
  }

  table[num_specs].type = SS_CIRCLE;
  table[num_specs].cx = cx;
  table[num_specs].cy = cy;
  table[num_specs].cradius = cradius;

  num_specs++;

}
//----------------------------------------------------------------------------
int simstat::call_add_spec_circle(int argc, char *argv[]) {

  if (argc!=3) return -1;

  add_spec_circle(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]));

  return 1;

}

//----------------------------------------------------------------------------
//	add_spec_sqr() 
//	  - add spec for a square area centered at cx, cy
//----------------------------------------------------------------------------
void simstat::add_spec_sqr(int cx, int cy, int radius){

  if (num_specs >= max_specs) {
	warning("simstat::add_spec_sqr","max specs exceeded");
	return; 
  }

  table[num_specs].type = SS_SQR;
  table[num_specs].cx = cx;
  table[num_specs].cy = cy;
  table[num_specs].radius = radius;

  num_specs++;

}
//----------------------------------------------------------------------------
int simstat::call_add_spec_sqr(int argc, char *argv[]) {

  if (argc!=3) return -1;

  add_spec_sqr(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]));

  return 1;

}


//----------------------------------------------------------------------------
//	add_spec_bbox() 
//	  - add spec for a bounded box area centered
//----------------------------------------------------------------------------
void simstat::add_spec_bbox(int lowx, int lowy, int highx, int highy){

  if (num_specs >= max_specs) {
	warning("simstat::add_spec_bbox","max specs exceeded");
	return; 
  }

  table[num_specs].type = SS_BBOX;
  table[num_specs].lowx = lowx;
  table[num_specs].lowy = lowy;
  table[num_specs].highx = highx;
  table[num_specs].highy = highy;

  num_specs++;

}
//----------------------------------------------------------------------------
int simstat::call_add_spec_bbox(int argc, char *argv[]) {

  if (argc!=4) return -1;

  add_spec_bbox(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]), atoi(argv[4]));

  return 1;

}


//----------------------------------------------------------------------------
//	sum_area() 
//	  - sum activity in the area specified by the table entry
//---------------------------------------------------------------------------
REAL simstat::sum_area(int idx) {

  REAL sum = (REAL) 0;
  int  buf_num = 0;	// temporarily
  int  x1,y1,x2,y2;

  switch(table[idx].type) {

	case SS_CIRCLE:
		
  		for (int i=0; i<cols; ++i) { 
    		   for (int j=0; j<rows; ++j) {

			if ( distance(i,j,table[idx].cx,table[idx].cy)
			     <= 
			     table[idx].cradius
			   ) 
        			sum += buffer[buf_num][i][j];

    		   }
  		}

		break;

	case SS_SQR:

		x1 = MAX(table[idx].cx-table[idx].radius,0);
		y1 = MAX(table[idx].cy-table[idx].radius,0);
		x2 = MIN(table[idx].cx+table[idx].radius,cols-1);
		y2 = MIN(table[idx].cy+table[idx].radius,rows-1);

  		for (int i=x1; i<=x2; ++i) 
    		   for (int j=y1; j<=y2; ++j) 
        		sum += buffer[buf_num][i][j];

		break;

	case SS_BBOX:

		x1 = MAX(table[idx].lowx,0);
		y1 = MAX(table[idx].lowy,0);
		x2 = MIN(table[idx].highx,cols-1);
		y2 = MIN(table[idx].highy,rows-1);
		
  		for (int i=x1; i<=x2; ++i) 
    		   for (int j=y1; j<=y2; ++j) 
        		sum += buffer[buf_num][i][j];

		break;

  }

  return sum;

}

