//----------------------------------------------------------------------------
// 
//	file: simstat.h
// 
//	function: gather various stats from a REAL matrix, over time
//			only works for scalar types
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Oct 30 23:54:02 CST 1998
// 
//	$Revision: 1.3 $ $Date: 2001/11/30 20:24:15 $
//	$Log: simstat.h,v $
//	Revision 1.3  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.2  2000/02/08 11:25:21  yschoe
//	*** empty log message ***
//
//	Revision 1.1  1999/02/09 07:42:25  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#ifndef __SIMSTAT_H
#define __SIMSTAT_H

#include <math.h>
#include "real.h"
#include "util.h"

enum SS_TYPE { SS_CIRCLE, SS_SQR, SS_BBOX };

typedef struct {

	SS_TYPE type;		// type
	int cx, cy, radius;	// SS_SQR
	int cradius;		// SS_CIRCLE
	int lowx, lowy, highx, highy; // SS_BBOX

} SS_Entry;

class simstat {

private:

  // The buffer is used to gather a number of matrix dumps
  // for calculaing statistics such as cross/auto correlation, etc.
  // -- ignore this for the time being
  REAL ***buffer;	// buffer[time][col][row]
  int  buffer_head;
  int  buffer_tail;
  int  buffer_size;

  int  cols, rows;
  int  max_specs;
  int  num_specs;

  SS_Entry *table;	// table to keep area specs to gather stat from

public:

  // some of these routines may
  //	1. copy

  simstat(REAL** buf, int cols, int rows, int max_specs=1);
 ~simstat();

  // sum activity in rectanble (x1,y1) to (x2,y2).
  REAL sum_area(int x1, int y1, int x2, int y2, int buf_num=0);

  // sum activity in the area specs of the table.
  REAL sum_area(int idx);
  int  get_num_specs() { return num_specs; }

  // add area spec 
  void	add_spec_circle(int cx, int cy, int cradius);
  void	add_spec_sqr(int cx, int cy, int radius);
  void  add_spec_bbox(int lowx, int lowy, int highx, int highy);

  // simenv interface
  int	call_add_spec_circle(int argc, char *argv[]);
  int	call_add_spec_sqr(int argc, char *argv[]);
  int	call_add_spec_bbox(int argc, char *argv[]);

  REAL  distance(int x1, int y1, int x2, int y2) {
		return (REAL) sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
  }

  int get_cx(int idx) { return table[idx].cx; }
  int get_cy(int idx) { return table[idx].cy; }
  int get_radius(int idx) { return table[idx].radius; }
  int get_cradius(int idx) { return table[idx].cradius; }

  SS_TYPE get_type(int idx) { return table[idx].type; };
  int get_lowx(int idx) { return table[idx].lowx; };
  int get_lowy(int idx) { return table[idx].lowy; };
  int get_highx(int idx) { return table[idx].highx; };
  int get_highy(int idx) { return table[idx].highy; };
 

};

#endif

