//----------------------------------------------------------------------------
// 
//	file: ppmio.cc
// 
//	function: ppm input output routines 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sat Nov  7 04:07:05 CST 1998
// 
//	$Revision: 1.4 $ $Date: 2000/02/08 11:25:21 $
//	$Log: ppmio.cc,v $
//	Revision 1.4  2000/02/08 11:25:21  yschoe
//	*** empty log message ***
//
//	Revision 1.3  2000/01/30 14:56:21  yschoe
//	*** empty log message ***
//
//	Revision 1.2  1999/02/09 07:42:25  yschoe
//	hsv mode added
//
//	Revision 1.1  1998/11/17 12:51:24  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------

#include "ppmio.h"

//----------------------------------------------------------------------------
//	ppmio() 
//	  - Constructor
//	  - Do not allocate matrix. Just keep a pointer to the
//	    target matrix to dump
//----------------------------------------------------------------------------
// matrix
ppmio::ppmio(REAL **mat, int cols, int rows, int colors=256) {

  ppmio::colors = colors;
  ppmio::cols = cols;
  ppmio::rows = rows;
  ppmio::mat 	= mat;

  type = PPM_REAL;

  color_tab = new struct color_t [colors];
  gray_tab = new struct color_t [colors];
  hsv_tab = new struct color_t [colors];
  hsv_tab2 = new struct color_t [colors];

  init_color_tab(colors);

}
//----------------------------------------------------------------------------
// integer matrix
ppmio::ppmio(int **mat, int cols, int rows, int colors=256) {

  ppmio::colors = colors;
  ppmio::cols = cols;
  ppmio::rows = rows;
  ppmio::mat 	= new REAL*[cols];
  for (int i=0; i<cols; ++i) {
	ppmio::mat[i] = new REAL[rows];
  }
  for (int i=0; i<cols; ++i) {
  	for (int j=0; j<rows; ++j) {
		ppmio::mat[i][j] = mat[i][j];
	}
  }
  
  type = PPM_INT;

  color_tab = new struct color_t [colors];
  gray_tab = new struct color_t [colors];
  hsv_tab = new struct color_t [colors];
  hsv_tab2 = new struct color_t [colors];

  init_color_tab(colors);

}


//----------------------------------------------------------------------------
// vector
ppmio::ppmio(REAL *vec, int cols, int rows, int colors=256) {

  ppmio::colors = colors;
  ppmio::cols = cols;
  ppmio::rows = rows;
  ppmio::mat 	= new REAL* [1];
  ppmio::mat[0] = vec;

  color_tab = new ctab_type [colors];

  init_color_tab(colors);

}

//----------------------------------------------------------------------------
//	~ppmio() 
//	  - Destructor
//----------------------------------------------------------------------------
ppmio::~ppmio() {

  delete [] color_tab; 
  delete [] gray_tab; 

  if (type == PPM_INT) {
	for (int i=0; i<cols; ++i) delete [] mat[i];
	delete [] mat;
  }

}

//----------------------------------------------------------------------------
//	init_color_tab() 
//	  - initialize color map table
//----------------------------------------------------------------------------
void ppmio::init_color_tab(int colors) {

 int rangeby3, multiplier, gray_multiplier, i ;

 rangeby3   = (int) (colors/3);
 multiplier = (int) (colors/rangeby3);

 for(i= 0; i<colors; i++ ) {

        color_tab[i].green = 0;
        color_tab[i].blue  = 0;

        if ( i < rangeby3 ){     // black --> red 

                color_tab[i].red = i*multiplier;

        } else {

                color_tab[i].red = 256-1;

                if ( i < 2*rangeby3){ // red --> yellow 

                        color_tab[i].green = (i-rangeby3)*multiplier;

                } else {  // yellow --> white 

                        color_tab[i].green = 256-1;
                        color_tab[i].blue  = (i - 2*rangeby3)*multiplier;

                }

        }

	hsvrgb::hsv2rgb(i/(REAL)(colors-1), 1.0, 1.0, 
		&hsv_tab[i].red, 
		&hsv_tab[i].green, 
		&hsv_tab[i].blue);

	hsvrgb::hsv2rgb((colors-i)*5.0/6.0/(REAL)(colors-1), 1.0, 1.0, 
		&hsv_tab2[i].red, 
		&hsv_tab2[i].green, 
		&hsv_tab2[i].blue);


	gray_tab[i].red = gray_tab[i].green = gray_tab[i].blue = i;
  }

}

//----------------------------------------------------------------------------
//	ppmwrite() 
//	  - write matrix to a ppm file
//----------------------------------------------------------------------------
void ppmio::ppmwrite(char *filename, REAL arg_min=0.0, REAL arg_max=0.0, 
	PPM_COLOR_MODE m=PPM_RGB) {

  FILE  *fp;
  REAL 	min = 9999999; 
  REAL 	max = -9999999;

  if (m==PPM_HSV2) {
	hsv_tab[0].red	= 255;
	hsv_tab[0].green = 255;
	hsv_tab[0].blue	= 255;
  }

  if (m==PPM_HSV3) {
	hsv_tab2[0].red	= 255;
	hsv_tab2[0].green = 255;
	hsv_tab2[0].blue	= 255;
  }

  // calc min and max if necessary
  if (REAL_EQ((REAL)arg_min,0.0) && REAL_EQ((REAL)arg_max,0.0)) {

	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      max = MAX(max,mat[i][j]);
	      min = MIN(min,mat[i][j]);
	   }
	}

  } else {

	min = arg_min;
	max = arg_max;

  }
 
  // open output file 
  if (!(fp=fopen(filename,"wb"))) {
        error_exit("ppmio::ppmwrite", "cannot create/write file");
  }

  // print header info
  fprintf(fp,"P6\n");
  fprintf(fp,"# %s : created by ppmio class : max = %f, min = %f \n",
		filename, (REAL)max, (REAL)min);
  fprintf(fp,"%d %d\n",cols, rows);
  fprintf(fp,"%d\n",256-1);

  // dump pixels
  REAL   diff = max-min;
  for (int j=rows-1; j>=0; --j) {
     for (int i=0; i<cols; ++i) {

	unsigned char r, g, b;

	int index = (int)((mat[i][j]-min)*(colors-1)/(REAL)diff);

	if (m == PPM_RGB) {

	   r = color_tab[index].red, 
	   g = color_tab[index].green,
           b = color_tab[index].blue;

	} else if (m == PPM_HSV || m == PPM_HSV2) {
	
	   r = hsv_tab[index].red, 
	   g = hsv_tab[index].green,
           b = hsv_tab[index].blue;

	   //hsvrgb::hsv2rgb((mat[i][j]-min)/(REAL)diff, 1.0, 1.0, &r, &g, &b);	

	} else if (m==PPM_HSV3) {
	
	   r = hsv_tab2[index].red, 
	   g = hsv_tab2[index].green,
           b = hsv_tab2[index].blue;

	} else if (m == PPM_GRAY) {

	   r = gray_tab[index].red, 
	   g = gray_tab[index].green,
           b = gray_tab[index].blue;

	} else if (m == PPM_GRAY2) {

	   int index = (mat[i][j]<(REAL)(diff/2))
			? (int)((mat[i][j]-min)*(colors-1)/(REAL)(diff/2)) 
			: (int)(-(mat[i][j]-min)*(colors-1)/(REAL)(diff/2)
				+2*(colors-1));

	   r = gray_tab[index].red, 
	   g = gray_tab[index].green,
           b = gray_tab[index].blue;

	}

	fwrite(&r,1,1,fp);
	fwrite(&g,1,1,fp);
	fwrite(&b,1,1,fp);
	//fprintf(fp, "%d %d %d  ", r, g, b);
	//if ((i+1)%4==0) fprintf(fp,"\n");
     }
  }

  fclose(fp);

}

//----------------------------------------------------------------------------
//	ppmheader() 
//	  - read ppm header
//----------------------------------------------------------------------------
int ppmio::ppmheader(FILE *fp, int *retcols, int *retrows, int *retcolors) {

  char buffer[512];
  
  // negative numbers mean these are uninitialized
  int  ppmcols=-1, ppmrows=-1;
  int  ppmcolors=-1;

  // assume that first line is magic number
  if (!fgets(buffer,512,fp)) {
	warning("ppmio::ppmread","error reading file");
	return 0;
  }
  if (strncmp(buffer,"P3",2)!=0) {
	warning("ppmio::ppmread","this is not a ppm file - check magic number");
	return 0;
  }

  while (!feof(fp)) {

	// read single word
  	if (!fscanf(fp,"%s",buffer)) {
		warning("ppmio::ppmread","error reading file");
		return 0;
  	}

	// Check if it is a comment : naive comment -- just check for '#'
  	// at the beginning of line
	if (buffer[0]=='#') {
		fgets(buffer,256,fp); continue;
   	}

	// if image size has not been read read it.
	if (ppmcols<0) {
		sscanf(buffer,"%d",&ppmcols); continue;
	}

	if (ppmrows<0) {
		sscanf(buffer,"%d",&ppmrows); continue;
	}

	// if colors has not been read, read it
	if (ppmcolors<0) {
		sscanf(buffer,"%d",&ppmcolors); break; // continue;
	}

  }

  *retcols = ppmcols;
  *retrows = ppmrows;
  *retcolors = ppmcolors;

  return 1;

}

//----------------------------------------------------------------------------
//	ppmread() 
//	  - read matrix from ppm file
//	  - Assume it is in :
//		1. color
//		2. in ASCII mode
//		3. ppm file starts with 'P3\n'
//		4. ppm data cluster does not contain comments
//----------------------------------------------------------------------------
void ppmio::ppmread(char *filename) {

  FILE *fp = fopen(filename,"r");
  char buffer[512];
  
  // negative numbers mean these are uninitialized
  int  ppmcols=-1, ppmrows=-1;
  int  ppmcolors=-1;

  ppmheader(fp,&ppmcols,&ppmrows,&ppmcolors);

  // check if ppm file can fit into the allocated memory, etc.
  if (ppmcols > cols || ppmrows > rows || (ppmcolors+1) > colors) {
	warning("ppmio::ppmread","image larger than allocated resources");
	return;
  } 

  // debug: cout << ppmcols <<"x" << ppmrows <<" colors = " <<ppmcolors << endl;

  // read in data (R G B values, and fill matrix)
  // 	- read it in row major 
  int count = 0;
  int R, G, B;
  while (!feof(fp)) {

	fscanf(fp,"%d %d %d ",&R, &G, &B);
	
	// scale it to a value between 0.0 an 1.0
	// Currently, it only supports 0.0 and 1.0 -- 0 0 0 and 255 255 255
	if (R==0 && G==0 && B==0) {
		mat[count%ppmcols][count/ppmcols] = 0.0;
	} else {	
		mat[count%ppmcols][count/ppmcols] = 1.0;
  	}
	count ++;
  }

  fclose(fp);

}


//----------------------------------------------------------------------------
//	ppmcolormap() 
//	  - dump current colormap in octave format : n row x 3 column matrix
//----------------------------------------------------------------------------
void ppmio::ppmcolormap(char *filename, PPM_COLOR_MODE mode=PPM_RGB) {

 struct color_t *ctab = NULL;
 char *name[3] = { "graymap", "rgbmap", "hsvmap"};
 char nameidx=0;
 FILE *fp;
 int colors=256;

 switch (mode) {

	case PPM_GRAY : ctab = &(gray_tab[0]); nameidx=0; break;
	case PPM_RGB  : ctab = &(color_tab[0]); nameidx=1; break;
	case PPM_HSV  : ctab = &(hsv_tab[0]); nameidx=2; break;
	default: error_exit("ppmio::ppmcolormap", "unknown mode");
 }

 if ((fp = fopen(filename,"w"))==0) {
        error_exit("ppmio::ppmcolormap", "cannot create/write file");
 }

 // Print header
 fprintf(fp,"#\n# name: %s\n# rows: %d\n# cols: %d\n", 
	name[nameidx], colors, 3);
 
 for(int i= 0; i<colors; i++ ) {
	fprintf(fp,"%f %f %f\n",ctab[i].red/(REAL)colors, 
		ctab[i].green/(REAL)colors,
		ctab[i].blue/(REAL)colors);
 }
 fclose(fp);

}

//----------------------------------------------------------------------------
//	ppmansi() 
//	  - display ppm in ansi
//----------------------------------------------------------------------------
void ppmio::ppmansi(FILE *fp=stdout, REAL min=0.0, REAL max=0.0, 
		PPM_COLOR_MODE m=PPM_RGB) {

  char buff[256];
  ppmwrite(".tmp.ppm",min,max,m);
  FILE *pp;
  pp = popen("/usr/bin/ppm2ansi .tmp.ppm","r");
  while (!feof(pp)) {
	fgets(buff,256,pp);
	fprintf(fp,"%s",buff);
  }
  pclose(pp);

}
  

