//----------------------------------------------------------------------------
// 
//	file: tool_window.cc
// 
//	function: create a popup with some tools as a base to 
//		host other generic window types 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Tue Jul  7 07:02:29 CDT 1998
// 
//	$Revision: 1.7 $ $Date: 2001/11/30 20:24:15 $
//	$Log: tool_window.cc,v $
//	Revision 1.7  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.6  1999/02/09 07:42:25  yschoe
//	test tool_window::BRelease_CB
//
//	Revision 1.5  1998/10/26 04:00:45  yschoe
//	fixed bug in destructor -- buttons were being deleted even when they
//	were not allocated in the beginning (with addbutton==0 at the constructor
//	call)
//
//	Revision 1.4  1998/10/16 10:08:04  yschoe
//	fixed minor bug. reduce residual area (for buttons, etc) a bit.
//
//	Revision 1.3  1998/10/05 19:48:46  yschoe
//	added refresh and close button
//
//	Revision 1.2  1998/09/04 03:36:37  yschoe
//	fixed all warnings generated by -Wall
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------

#include <window.h>
#include <iostream.h>
#include "tool_window.h"


//----------------------------------------------------------------------------
//	tool_window() 
//	  - Constructor
//----------------------------------------------------------------------------
tool_window::tool_window(char *name, int w, int h, int x, int y, 
	int addbuttons = 0) : main_window(name, w, h+20*addbuttons, 1, x, y) {

  button_flag = addbuttons;
  selection_mask |= ButtonReleaseMask;
  init (addbuttons);
  // child window goes here : new class (*this, .... )
  unmapped = True;
}

//----------------------------------------------------------------------------
//	~tool_window() 
//	  - destructor
//----------------------------------------------------------------------------
tool_window::~tool_window() {

  if (button_flag) {
	delete refresh_button;
	delete close_button;
  }
}

//----------------------------------------------------------------------------
//	toggle() 
//	  - Toggle mapped/unmapped state
//----------------------------------------------------------------------------
void tool_window::toggle() {

  if (unmapped) 
	RealizeChildren(); 
  else 
	Unmap();

  unmapped = ! unmapped;

}

//----------------------------------------------------------------------------
//	init() 
//	  - init tools
//----------------------------------------------------------------------------
void tool_window::init(int addbuttons) {

  // Set up default scrollbar, etc
  num_sbars = 0;

/*
  scrollbar[0] = new scrollbar(*this, (void (*)(void *))&scrollbar_callback,
		this, width/2,20,
		0,height-40,0.1,10.0,0.1,"%.1lf");
  old_sb_value[0] = (scrollbar[0]->value);
  num_scrollbars++;

  scrollbar[1] = new scrollbar(*this, (void (*)(void *))&scrollbar_callback, 
		this, width/2,20,
		0,height-20,0.1,10.0,0.1,"%.1lf");
  old_sb_value[1] = (scrollbar[1]->value);
  num_scrollbars++;

  cback[0]  = new callback_button(*this,"dump",(void (*)())&cback_callback,
		width/4,20,width*3/4,height-20);
*/

  if (addbuttons) {
     // refresh_button = new callback_button(*this,"refresh",(void (*)())&refresh,
     //		width/4,20,width/2,height-20);
     close_button = new unmap_button(*this,"close",width/4,20,
		width*3/4,height-20);
  }

}
   
/* : disabled for now
//----------------------------------------------------------------------------
//	add_tool() 
//	  - add tools 
//----------------------------------------------------------------------------
void tool_window::add_tool(int tool_id, char *name, int type, int w, int h,
	int x, int y, (void (*callback)()), char *stat_str) {

  if (tool_id > MAX_G) {
	error_exit("tool_window::add_tool","id too large");
  }

  switch ((ToolType)type) {
	case SCROLL: 
		scrollbar[tool_id]=new scrollbarbar(*this,callback,w,h,x,y,stat_str);
		sb_value[tool_id] = &(scrollbar[tool_id]->value);
		break;
	case CALLBACK:
		cback[tool_id]=new callback_button(*this,name,callback,w,h,x,y);
		break;
	case UNMAP:
		new unmap_button(*this,name,w,h,x,y);
		break;
	default:
		err_exit("tool_window::add_tool","unknown ToolType");
  }

}
*/

//----------------------------------------------------------------------------
//	scrollbar_callback() 
//	  - scrollbarbar callback function
//----------------------------------------------------------------------------
void tool_window::sbar_callback(tool_window *tw) {

  cout << "scrollbar_callback\n";
  for (int i=0; i<num_sbars; ++i) {
	if (fabs(tw->old_sb_value[i]-tw->sbar[i]->value)<0.0000001) {
		cout << "id = " << i << endl;
		tw->old_sb_value[i] = tw->sbar[i]->value;
        }
  }
  
}

//----------------------------------------------------------------------------
//	cback_callback() 
//	  - cback button callback function
//----------------------------------------------------------------------------
void tool_window::cback_callback() {

  // cout << "cback_callback\n";
  // override this 
}


//----------------------------------------------------------------------------
//	add_scrollbar() 
//	  - add scroll bar to this window
//----------------------------------------------------------------------------
void tool_window::add_sbar(void (*cback_func)(), 
	int width, int height,
	int x, int y, double start, double stop, double initial, 
	char *fmt_str) {

  sbar[num_sbars++] = new scrollbar(*this, cback_func,
		width,height,x,y,start,stop,initial,fmt_str);

}

//----------------------------------------------------------------------------
//	refresh() 
//	  - refresh display
//----------------------------------------------------------------------------
void tool_window::refresh() {

  redraw();
  XFlush(display);

} 

void tool_window::BRelease_CB(XButtonEvent ev) {

  cout << "tool_window" << ev.x << "," << ev.y << endl;

}
