//----------------------------------------------------------------------------
// 
//	file: correl.c
// 
//	function: calculate auto and cross correlation of time series 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Nov  6 03:50:40 CST 1998
// 
//	$Revision: 1.1 $ $Date: 1998/11/17 12:50:32 $
//	$Log: correl.cc,v $
//	Revision 1.1  1998/11/17 12:50:32  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#include <iostream.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <nnview/real.h>

enum CORREL_TYPE {AUTO, CROSS};

//----------------------------------------------------------------------------
//	autocorrel() 
//	  - return auto correlation of timeseries and return r_k in dest.
//----------------------------------------------------------------------------
REAL autocorrel(REAL *dest, REAL *series, int size) {

  // calculate mean
  REAL sum = 0.0;
  for (int i=0; i<size; ++i) {
	sum += series[i];
  }
  REAL mean = sum/(REAL)size;

  // calculate Sum((x_t-mean)^2) : commonly used for all r_k
  REAL sumsqr = 0.0;
  for (int i=0; i<size; ++i) {
	sumsqr += (series[i]-mean)*(series[i]-mean);
  }

  // calculate r_k 
  for (int k=0; k<size; ++k) {
     dest[k] = 0.0;
     for (int t=0; t<size-k; ++t) {
	dest[k] += (series[t]-mean)*(series[t+k]-mean)/sumsqr;
     }
  }
  return 0.0;
}

//----------------------------------------------------------------------------
//	crosscorrel() 
//	  - return auto correlation of timeseries and return r_k in dest.
//----------------------------------------------------------------------------
REAL crosscorrel(REAL *dest, REAL *series1, REAL *series2, int size) {

  // calculate mean
  REAL sum1 = 0.0, sum2 = 0.0;
  for (int i=0; i<size; ++i) {
	sum1 += series1[i];
	sum2 += series2[i];
  }
  REAL mean1 = sum1/(REAL)size;
  REAL mean2 = sum2/(REAL)size;

  // calculate Sum((x_t-mean)^2) : commonly used for all r_k
  REAL sumsqr1 = 0.0;
  REAL sumsqr2 = 0.0;
  for (int i=0; i<size; ++i) {
	sumsqr1 += (series1[i]-mean1)*(series1[i]-mean1);
	sumsqr2 += (series2[i]-mean2)*(series2[i]-mean2);
  }

  // calculate r_k 
  for (int k=0; k<size; ++k) {
     dest[k] = 0.0;
     for (int t=0; t<size-k; ++t) {
	dest[k] += (series1[t]-mean1)*(series2[t+k]-mean2)
		/ (sqrt(sumsqr1)*sqrt(sumsqr2));
     }
  }
  return 0.0;;

}


//----------------------------------------------------------------------------
//	main() 
//----------------------------------------------------------------------------
int main (int argc, char *argv[]) {

  REAL *series1;
  REAL *series2;
  REAL *correl;
  CORREL_TYPE type;

  if (argc!=3) {
     cout << "usage: correl auto|cross <samples>" 
          << "       input: stdin " << endl;
     exit(0);
  }

  if (strcmp(argv[1],"auto")==0) {
     type = AUTO;
  } else if (strcmp(argv[1],"cross")==0) {
     type = CROSS;
  }

  int  size = atoi(argv[2]);

  series1 = new REAL [size];
  series2 = new REAL [size];
  correl = new REAL [size];

  for (int i=0; i<size; ++i) {
     if (type==CROSS) 
	fscanf(stdin,"%f %f",&series1[i],&series2[i]);
     else if (type==AUTO) 
	fscanf(stdin,"%f",&series1[i]);
  }

  if (type==CROSS) 
     crosscorrel(correl,series1,series2,size);
  else if (type==AUTO)
     autocorrel(correl,series1,size);

  for (int i=0; i<size; ++i) {
     if (type==AUTO) {
     	cout << (i+0.5) << " " << correl[i] << endl;
     } else {
     	cout << (i) << " " << correl[i] << endl;
     }
  }

}  
